<%@ WebHandler Language="C#" Class="redirect" %>

using System;
using System.Web;
using System.Xml;
using System.IO;
using System.Text;
using System.Globalization;

public class redirect : IHttpHandler
{

  public void ProcessRequest(HttpContext context)
  {
    string url = context.Request.QueryString["url"];
    if (!String.IsNullOrEmpty(url))
    {
      RegisterClick(url);
      PermanentRedirect(url);
    }

    string stats = context.Request.QueryString["stats"];
    if (!String.IsNullOrEmpty(stats) && stats.Equals("true", StringComparison.OrdinalIgnoreCase))
    {
      ShowStats();
    }
  }

  /// <summary>
  /// Performs a 301 redirection to the specified URL.
  /// </summary>
  private void PermanentRedirect(string url)
  {
    HttpContext.Current.Response.StatusCode = 301;
    HttpContext.Current.Response.AppendHeader("location", url);
    HttpContext.Current.Response.End();
  }

  /// <summary>
  /// Used to thread safe the file operations
  /// </summary>
  private static object _SyncRoot = new object();
  
  /// <summary>
  /// The XML filed used to save and retrieve the stats.
  /// </summary>
  private static XmlDocument _Document;
  
  /// <summary>
  /// The relative path of the XML file.
  /// </summary>
  private const string FILENAME = "~/app_data/redirection.xml";

  private void RegisterClick(string url)
  {
    string fileName = HttpContext.Current.Server.MapPath(FILENAME);

    lock (_SyncRoot)
    {
      if (_Document == null)
        CreateDocument(fileName);

      string address = HttpUtility.HtmlEncode(url);
      XmlNode node = _Document.SelectSingleNode("urls/url[@address='" + address + "']");
      if (node == null)
      {
        AddNewUrl(address);
      }
      else
      {
        int count = int.Parse(node.InnerText, CultureInfo.InvariantCulture);
        node.InnerText = (count + 1).ToString();
      }

      _Document.Save(fileName);
    }
  }

  /// <summary>
  /// Adds a new Url to the XmlDocument.
  /// </summary>
  private static void AddNewUrl(string address)
  {
    XmlNode newNode = _Document.CreateElement("url");
    
    XmlAttribute attrAddress = _Document.CreateAttribute("address");
    attrAddress.Value = address;
    newNode.Attributes.Append(attrAddress);

    XmlAttribute attrDate = _Document.CreateAttribute("date");
    attrDate.Value = DateTime.Now.ToString();
    newNode.Attributes.Append(attrDate);
    
    newNode.InnerText = "1";
    _Document.ChildNodes[1].AppendChild(newNode);
  }

  /// <summary>
  /// Creates the XML file for first time use.
  /// </summary>
  private static void CreateDocument(string fileName)
  {
    _Document = new XmlDocument();
    
    if (!File.Exists(fileName))
    {
      using (XmlWriter writer = XmlWriter.Create(fileName))
      {
        writer.WriteStartDocument(true);
        writer.WriteStartElement("urls");
        writer.WriteEndElement();
      }
    }

    _Document.Load(fileName);
  }

  /// <summary>
  /// Shows the collected statistics on the clicks.
  /// </summary>
  private void ShowStats()
  {
    StringBuilder sb = new StringBuilder();
    sb.AppendLine("<table style=\"font:10px verdana\">");
    sb.AppendLine("\t<tr>");
    sb.AppendLine("\t\t<th>Address</th>");
    sb.AppendLine("\t\t<th>Creation date</th>");
    sb.AppendLine("\t\t<th>Clicks</th>");
    sb.AppendLine("\t</tr>");

    if (_Document == null)
    {
      string fileName = HttpContext.Current.Server.MapPath(FILENAME);
      CreateDocument(fileName);
    }

    foreach (XmlNode node in _Document.SelectNodes("urls/url"))
    {
      sb.AppendLine("\t<tr>");
      sb.AppendLine("\t\t<td>" + node.Attributes["address"].InnerText + "</td>");
      sb.AppendLine("\t\t<td>" + DateTime.Parse(node.Attributes["date"].InnerText).ToString("d. MMMM yyyy HH:mm:ss") + "</td>");
      sb.AppendLine("\t\t<td style=\"text-align:center\">" + node.InnerText + "</td>");
      sb.AppendLine("\t</tr>");
    }

    sb.AppendLine("</table>");
    HttpContext.Current.Response.Write(sb.ToString());
  }

  public bool IsReusable
  {
    get { return false; }
  }

}